/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "wx/wx.h"
#include <wx/stdpaths.h>
#include <wx/config.h>

#include "main.h"

#include "GUI/MainFrame.h"
#include "Midi/Players/PlatformMidiManager.h"
#include "Images/ImageProvider.h"

#include <iostream>

#include "Config.h"	

IMPLEMENT_APP(AriaMaestosa::wxWidgetApp)

#ifdef _MORE_DEBUG_CHECKS
#include "LeakCheck.h"
#endif

namespace AriaMaestosa {
	
MainFrame* frame = NULL;
wxLocale* locale;

MainFrame* getMainFrame(){ assert(frame != NULL); return frame; }

wxConfig* prefs;

bool wxWidgetApp::OnInit()
{

	prefs = (wxConfig*) wxConfig::Get();
	
	long language;
	
	if(! prefs->Read("lang", &language) ){
		language = wxLANGUAGE_DEFAULT;
		std::cout << "failed to read prefs" << std::endl;
	}
	//std::cout << "language: " << language << std::endl;
	
	locale = new wxLocale( language, wxLOCALE_CONV_ENCODING );
	// wxLANGUAGE_DEFAULT
	// wxLANGUAGE_ENGLISH
	// wxLANGUAGE_FRENCH
	
	locale->AddCatalog("internat");
	
    PlatformMidiManager::initMidiPlayer();

    frame=new MainFrame();
        
    return true;
}


int wxWidgetApp::OnExit()
{
	
#ifdef _MORE_DEBUG_CHECKS
	MemoryLeaks::checkForLeaks();
#endif
    return 0;
}


void wxWidgetApp::MacOpenFile(const wxString &fileName)
{

	if(fileName.EndsWith("aria"))
	{
		frame->loadAriaFile( fileName );	
	}
	else if(fileName.EndsWith("mid") or fileName.EndsWith("midi"))
	{
		frame->loadMidiFile( fileName );	
	}
	else
	{
		wxMessageBox(wxString("Unknown file type: ") + fileName);	
	}
	
}

}
